package com.notuvy.net.webcache;

import com.notuvy.util.WallClock;
import org.apache.log4j.Logger;

import java.util.concurrent.TimeUnit;

/**
 *
 * User: Murali
 * Date: 6/29/12
 * Time: 6:19 AM
 * To change this template use File | Settings | File Templates.
 */
public class CacheQuery {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger("com.notuvy.net");


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final CachedWebProxy fCachedWebProxy;
    private final String fUrl;
    private long fExpiration;


    //------------------------------------------------------------
    //- Constructors

    protected CacheQuery(CachedWebProxy pCachedWebProxy, String pUrl, long pExpiration) {
        fCachedWebProxy = pCachedWebProxy;
        fUrl = pUrl;
        setExpiration(pExpiration, TimeUnit.MILLISECONDS);
    }

    //------------------------------------------------------------
    //- Gettors

    public String getUrl() {
        return fUrl;
    }

    public long getExpiration() {
        return fExpiration;
    }


    //------------------------------------------------------------
    //- Settors

    /**
     * Change the expiration of this particular query.
     *
     * @param pExpiration  The amount of time after which a download must be done.
     * @param pTimeUnit  The unit of time defining the expiration time amount.
     * @return  self for cascaded calls.
     */
    public CacheQuery setExpiration(long pExpiration, TimeUnit pTimeUnit) {
        fExpiration = pTimeUnit.toMillis(pExpiration);
        return this;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected boolean hasExpired(long pCreationTime) {
        return (getExpiration() == 0) || ((getExpiration() > 0) && (WallClock.currentTimeMillis() >= (pCreationTime + getExpiration())));
    }

    protected long computeExpirationTime(long pCreationTime) {
        long result = Long.MAX_VALUE;
        if (getExpiration() == 0) {
            result = pCreationTime;
        } else if (getExpiration() > 0) {
            result = pCreationTime + getExpiration();
        }
        return result;
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    /**
     * Perform the full lookup (which may come from the cache).
     *
     * @return  The result served from a local file.
     */
    public CacheResult lookup() {
        return fCachedWebProxy.lookup(this);
    }

    /**
     * Define the query to not be cached.  If re-queried, a download will always be done.
     *
     * @return  self for cascaded calls.
     */
    public CacheQuery setNoCache() {
        return setExpiration(0, TimeUnit.MILLISECONDS);
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    @Override
    public String toString() {
        return String.format("CacheQuery[%s][%d]", getUrl(), getExpiration());
    }


    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main


}
